from cone.app.browser import render_main_template
from cone.app.browser.utils import make_url
from cone.tile import Tile
from cone.tile import tile
from pyramid.i18n import TranslationStringFactory
from pyramid.i18n import get_localizer
from pyramid.view import view_config


_ = TranslationStringFactory('cone.fileupload')


I18N_MESSAGES = u"""
<script type="text/javascript">
    var fileupload_i18n_messages = new Object();
    fileupload_i18n_messages.uploadedBytes = '{uploaded_bytes}';
    fileupload_i18n_messages.acceptFileTypes = '{accept_file_types}';
</script>
"""


UPLOAD_TEMPLATE = u"""
<script id="template-upload" type="text/x-tmpl">
{{% for (var i=0, file; file=o.files[i]; i++) {{ %}}
    <tr class="template-upload fade">
        <td>
            <span class="preview"></span>
        </td>
        <td>
            <p class="name">{{%=file.name%}}</p>
            <strong class="error text-danger"></strong>
        </td>
        <td>
            <p class="size">{processing}</p>
            <div class="progress progress-striped active"
                 role="progressbar"
                 aria-valuemin="0"
                 aria-valuemax="100"
                 aria-valuenow="0">
                <div class="progress-bar progress-bar-success"
                     style="width:0%;">
                </div>
            </div>
        </td>
        <td>
            {{% if (!i && !o.options.autoUpload) {{ %}}
                <button class="btn btn-primary start" disabled>
                    <i class="glyphicon glyphicon-upload"></i>
                    <span>{start}</span>
                </button>
            {{% }} %}}
            {{% if (!i) {{ %}}
                <button class="btn btn-warning cancel">
                    <i class="glyphicon glyphicon-ban-circle"></i>
                    <span>{cancel}</span>
                </button>
            {{% }} %}}
        </td>
    </tr>
{{% }} %}}
</script>
"""


DOWNLOAD_TEMPLATE = u"""
<script id="template-download" type="text/x-tmpl">
{{% for (var i=0, file; file=o.files[i]; i++) {{ %}}
    <tr class="template-download fade">
        <td>
            <span class="preview">
                {{% if (file.thumbnailUrl) {{ %}}
                    <a href="{{%=file.url%}}"
                       title="{{%=file.name%}}"
                       download="{{%=file.name%}}"
                       data-gallery><img src="{{%=file.thumbnailUrl%}}"></a>
                {{% }} %}}
            </span>
        </td>
        <td>
            <p class="name">
                {{% if (file.url) {{ %}}
                    <a href="{{%=file.url%}}"
                       title="{{%=file.name%}}"
                       download="{{%=file.name%}}"
                       {{%=file.thumbnailUrl?'data-gallery':''%}}>
                        {{%=file.name%}}
                    </a>
                {{% }} else {{ %}}
                    <span>{{%=file.name%}}</span>
                {{% }} %}}
            </p>
            {{% if (file.error) {{ %}}
                <div>
                  <span class="label label-danger">{error}</span>
                  {{%=file.error%}}
                </div>
            {{% }} %}}
        </td>
        <td>
            <span class="size">{{%=o.formatFileSize(file.size)%}}</span>
        </td>
        <td>
            {{% if (file.deleteUrl) {{ %}}
                <button class="btn btn-danger delete"
                        data-type="{{%=file.deleteType%}}"
                        data-url="{{%=file.deleteUrl%}}"
                        {{% if (file.deleteWithCredentials) {{ %}}
                        data-xhr-fields='{{"withCredentials":true}}'
                        {{% }} %}}>
                    <i class="glyphicon glyphicon-trash"></i>
                    <span>{delete}</span>
                </button>
                <input type="checkbox" name="delete" value="1" class="toggle">
            {{% }} else {{ %}}
                <button class="btn btn-warning cancel">
                    <i class="glyphicon glyphicon-ban-circle"></i>
                    <span>{cancel}</span>
                </button>
            {{% }} %}}
        </td>
    </tr>
{{% }} %}}
</script>
"""


@tile(name='fileupload', path='fileupload.pt', permission='add')
class FileUploadTile(Tile):
    """Register this tile for specific context if jQuery file upload settings
    should be customized.
    """
    accept_file_types = ''  # e.g. /(\.|\/)(gif|jpe?g|png)$/i
    disable_image_preview = False
    disable_video_preview = False
    disable_audio_preview = False

    @property
    def i18n_messages(self):
        localizer = get_localizer(self.request)
        translate = localizer.translate
        return I18N_MESSAGES.format(
            uploaded_bytes=translate(
                _('uploaded_bytes',
                  default='Uploaded bytes exceed file size')),
            accept_file_types=translate(
                _('accept_file_types',
                  default='File type not allowed')),
        )

    @property
    def upload_template(self):
        localizer = get_localizer(self.request)
        translate = localizer.translate
        return UPLOAD_TEMPLATE.format(
            processing=translate(_('processing', default='Processing...')),
            start=translate(_('start', default='Start')),
            cancel=translate(_('cancel', default='Cancel')),
        )

    @property
    def download_template(self):
        localizer = get_localizer(self.request)
        translate = localizer.translate
        return DOWNLOAD_TEMPLATE.format(
            error=translate(_('error', default='Error')),
            delete=translate(_('delete', default='Delete')),
            cancel=translate(_('cancel', default='Cancel')),
        )

    @property
    def upload_url(self):
        return make_url(
            self.request,
            node=self.model,
            resource='fileupload_handle'
        )


@view_config(name='fileupload', permission='add')
def fileupload(model, request):
    """Fileupload as traversable view.
    """
    return render_main_template(model, request, 'fileupload')


# unknown mimetype marker
UNKNOWN_MIMETYPE = object()


@view_config(
    name='fileupload_handle',
    accept='application/json',
    renderer='json',
    permission='add')
class FileUploadHandle(object):
    """Abstract file upload handle.

    Subclass this object and register ``fileupload_handle`` tile for specific
    context.

    Methods ``read_existing`` and ``create_file`` are supposed to be
    implemented on deriving object.
    """

    def __init__(self, model, request):
        self.model = model
        self.request = request

    def __call__(self):
        result = dict()
        # initial reading
        if self.request.method == 'GET':
            result['files'] = self.read_existing()
        # file upload
        elif self.request.method == 'POST':
            result['files'] = list()
            for filedata in self.request.params.values():
                mimetype = filedata.type or UNKNOWN_MIMETYPE
                filename = filedata.filename
                stream = filedata.file
                res = self.create_file(stream, filename, mimetype)
                result['files'].append(res)
        return result

    def read_existing(self):
        """Read existing files in context and return a list of dicts containing
        client data.

        [
            {
                'name': 'pic.jpg',
                'size': 841946,
                'url': 'http://example.org/pic.jpg',
                'thumbnailUrl': 'http://example.org/thumbnail/pic.jpg',
                'deleteUrl': 'http://example.org/pic.jpg/filedelete_handle',
                'deleteType': 'GET',
            }
        ]
        """
        return []

    def create_file(self, stream, filename, mimetype):
        """Create file by uploaded stream and return client data dict.

        {
            'name': 'pic.jpg',
            'size': 841946,
            'url': 'http://example.org/pic.jpg',
            'thumbnailUrl': 'http://example.org/thumbnail/pic.jpg',
            'deleteUrl': 'http://example.org/pic.jpg/filedelete_handle',
            'deleteType': 'GET',
        }
        """
        return {
            'name': filename,
            'size': 0,
            'error': 'Abstract ``FileUploadHandle`` does not implement ' +
                     '``create_file``'
        }


@view_config(
    name='filedelete_handle',
    accept='application/json',
    renderer='json',
    permission='delete')
def filedelete_handle(model, request):
    """Delete file and return client data.

    {
        'files': [{
            'pic.jpg': True
        }]
    }
    """
    parent = model.parent
    name = model.name
    del parent[name]
    parent()
    return {'files': [{name: True}]}



from cgi import FieldStorage
from cone.app import testing
from cone.app.model import BaseNode
from cone.fileupload.browser.fileupload import DOWNLOAD_TEMPLATE
from cone.fileupload.browser.fileupload import filedelete_handle
from cone.fileupload.browser.fileupload import fileupload
from cone.fileupload.browser.fileupload import FileUploadHandle
from cone.fileupload.browser.fileupload import I18N_MESSAGES
from cone.fileupload.browser.fileupload import UPLOAD_TEMPLATE
from cone.tile import render_tile
from cone.tile.tests import TileTestCase
from pyramid.httpexceptions import HTTPForbidden
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Allow
from pyramid.security import Deny
from pyramid.security import Everyone
try:
    from StringIO import StringIO
except ImportError:  # pragma: no cover
    from io import StringIO
import sys
import unittest


class FileuploadLayer(testing.Security):

    def make_app(self, **kw):
        super(FileuploadLayer, self).make_app(**{
            'cone.plugins': 'cone.fileupload'
        })


ACL = [
    (Allow, 'role:manager', ['add', 'delete']),
    (Allow, Everyone, ['login']),
    (Deny, Everyone, ALL_PERMISSIONS)
]


class ContainerNode(BaseNode):
    __acl__ = ACL

    def __call__(self):
        pass


class File(BaseNode):
    __acl__ = ACL
    allow_non_node_childs = True


class ContainerFileUploadHandle(FileUploadHandle):

    def create_file(self, stream, filename, mimetype):
        file = self.model[filename] = File()
        file['body'] = stream.read()
        return {
            'name': filename,
            'size': len(file['body']),
            'url': '/{0}'.format(file.name),
            'deleteUrl': '/{0}/filedelete_handle'.format(file.name),
            'deleteType': 'GET',
        }

    def read_existing(self):
        files = list()
        for node in self.model.values():
            files.append({
                'name': node.name,
                'size': len(node['body']),
                'url': '/{0}'.format(node.name),
                'deleteUrl': '/{0}/filedelete_handle'.format(node.name),
                'deleteType': 'GET',
            })
        return files


class TestFileupload(TileTestCase):
    layer = FileuploadLayer()

    def test_default_templates(self):
        # Default i18n messages, upload and download templates
        self.checkOutput("""
        <script type="text/javascript">...</script>
        """, I18N_MESSAGES)

        self.checkOutput("""
        <script id="template-upload" type="text/x-tmpl">...</script>
        """, UPLOAD_TEMPLATE)

        self.checkOutput("""
        <script id="template-download" type="text/x-tmpl">...</script>
        """, DOWNLOAD_TEMPLATE)

    def test_fileupload_tile(self):
        container = ContainerNode(name='container')

        # Render fileupload tile unauthorized
        request = self.layer.new_request()
        err = self.expectError(
            HTTPForbidden,
            render_tile,
            container,
            request,
            'fileupload'
        )
        self.checkOutput("""
        Unauthorized: tile
        <cone.fileupload.browser.fileupload.FileUploadTile object at ...>
        failed permission check
        """, str(err))

        # Render fileupload tile authorized
        with self.layer.authenticated('manager'):
            res = render_tile(container, request, 'fileupload')
            self.assertTrue(res.find('<form id="fileupload"') > -1)

    def test_fileupload_view(self):
        # Traversable fileupload view
        container = ContainerNode(name='container')
        request = self.layer.new_request()

        with self.layer.authenticated('manager'):
            response = fileupload(container, request)

        self.assertTrue(response.text.startswith('<!DOCTYPE html'))
        self.assertTrue(response.text.find('<form id="fileupload"') > -1)

    def test_fileupload_handle(self):
        # Abstract file upload handle
        container = ContainerNode(name='container')
        request = self.layer.new_request()
        abstract_upload_handle = FileUploadHandle(container, request)

        # If request method is GET, existing files are read. Abstract
        # implementation returns empty result
        self.assertEqual(abstract_upload_handle(), {'files': []})

        # If request method is POST, a file upload is assumed
        filedata = FieldStorage()
        filedata.type = 'text/plain'
        filedata.filename = 'test.txt'
        filedata.file = StringIO('I am the payload')

        request.method = 'POST'
        request.params['file'] = filedata
        del request.params['_LOCALE_']

        res = abstract_upload_handle()
        self.assertEqual(res['files'][0]['name'], 'test.txt')
        self.assertEqual(res['files'][0]['size'], 0)
        self.assertEqual(
            res['files'][0]['error'],
            'Abstract ``FileUploadHandle`` does not implement ``create_file``'
        )

        # Concrete implementation of file upload handle
        upload_handle = ContainerFileUploadHandle(container, request)

        # Upload file
        res = upload_handle()
        self.assertEqual(res['files'], [{
            'url': '/test.txt',
            'deleteType': 'GET',
            'deleteUrl': '/test.txt/filedelete_handle',
            'name': 'test.txt',
            'size': 16
        }])

        self.checkOutput("""
        <class 'cone.fileupload.tests.ContainerNode'>: container
          <class 'cone.fileupload.tests.File'>: test.txt
            body: 'I am the payload'
        """, container.treerepr())

        # Read existing files
        request = self.layer.new_request()
        upload_handle = ContainerFileUploadHandle(container, request)
        self.assertEqual(upload_handle()['files'], [{
            'url': '/test.txt',
            'deleteType': 'GET',
            'deleteUrl': '/test.txt/filedelete_handle',
            'name': 'test.txt',
            'size': 16
        }])

        # Test file delete handle
        file = container['test.txt']
        request = self.layer.new_request()
        self.assertEqual(
            filedelete_handle(file, request),
            {'files': [{'test.txt': True}]}
        )

        self.checkOutput("""
        <class 'cone.fileupload.tests.ContainerNode'>: container
        """, container.treerepr())


def run_tests():
    from cone.fileupload import tests
    from zope.testrunner.runner import Runner

    suite = unittest.TestSuite()
    suite.addTest(unittest.findTestCases(tests))

    runner = Runner(found_suites=[suite])
    runner.run()
    sys.exit(int(runner.failed))


if __name__ == '__main__':
    run_tests()



from cone.app import cfg
from cone.app import main_hook
from cone.fileupload.browser import static_resources
import logging


logger = logging.getLogger('cone.fileupload')


@main_hook
def initialize_fileupload(config, global_config, settings):
    # application startup initialization

    # protected CSS
    cfg.css.protected.append('fileupload-static/css/jquery.fileupload.css')

    # protected JS
    cfg.js.protected.append('fileupload-static/js/vendor/tmpl.min.js')
    cfg.js.protected.append('fileupload-static/js/vendor/load-image.all.min.js')
    cfg.js.protected.append('fileupload-static/js/vendor/canvas-to-blob.min.js')
    cfg.js.protected.append('fileupload-static/js/vendor/jquery.blueimp-gallery.min.js')  # noqa
    cfg.js.protected.append('fileupload-static/js/jquery.iframe-transport.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload-process.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload-image.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload-audio.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload-video.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload-validate.js')
    cfg.js.protected.append('fileupload-static/js/jquery.fileupload-ui.js')
    cfg.js.protected.append('fileupload-static/js/cone.fileupload.js')

    # add translation
    config.add_translation_dirs('cone.fileupload:locale/')

    # static resources
    config.add_view(static_resources, name='fileupload-static')

    # scan browser package
    config.scan('cone.fileupload.browser')



# -*- coding: utf-8 -*-
#
# jQuery File Upload Plugin GAE Python Example 2.2.0
# https://github.com/blueimp/jQuery-File-Upload
#
# Copyright 2011, Sebastian Tschan
# https://blueimp.net
#
# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT
#

from __future__ import with_statement
from google.appengine.api import files, images
from google.appengine.ext import blobstore, deferred
from google.appengine.ext.webapp import blobstore_handlers
import json
import re
import urllib
import webapp2

WEBSITE = 'https://blueimp.github.io/jQuery-File-Upload/'
MIN_FILE_SIZE = 1  # bytes
MAX_FILE_SIZE = 5000000  # bytes
IMAGE_TYPES = re.compile('image/(gif|p?jpeg|(x-)?png)')
ACCEPT_FILE_TYPES = IMAGE_TYPES
THUMBNAIL_MODIFICATOR = '=s80'  # max width / height
EXPIRATION_TIME = 300  # seconds


def cleanup(blob_keys):
    blobstore.delete(blob_keys)


class UploadHandler(webapp2.RequestHandler):

    def initialize(self, request, response):
        super(UploadHandler, self).initialize(request, response)
        self.response.headers['Access-Control-Allow-Origin'] = '*'
        self.response.headers[
            'Access-Control-Allow-Methods'
        ] = 'OPTIONS, HEAD, GET, POST, PUT, DELETE'
        self.response.headers[
            'Access-Control-Allow-Headers'
        ] = 'Content-Type, Content-Range, Content-Disposition'

    def validate(self, file):
        if file['size'] < MIN_FILE_SIZE:
            file['error'] = 'File is too small'
        elif file['size'] > MAX_FILE_SIZE:
            file['error'] = 'File is too big'
        elif not ACCEPT_FILE_TYPES.match(file['type']):
            file['error'] = 'Filetype not allowed'
        else:
            return True
        return False

    def get_file_size(self, file):
        file.seek(0, 2)  # Seek to the end of the file
        size = file.tell()  # Get the position of EOF
        file.seek(0)  # Reset the file position to the beginning
        return size

    def write_blob(self, data, info):
        blob = files.blobstore.create(
            mime_type=info['type'],
            _blobinfo_uploaded_filename=info['name']
        )
        with files.open(blob, 'a') as f:
            f.write(data)
        files.finalize(blob)
        return files.blobstore.get_blob_key(blob)

    def handle_upload(self):
        results = []
        blob_keys = []
        for name, fieldStorage in self.request.POST.items():
            if type(fieldStorage) is unicode:
                continue
            result = {}
            result['name'] = re.sub(
                r'^.*\\',
                '',
                fieldStorage.filename
            )
            result['type'] = fieldStorage.type
            result['size'] = self.get_file_size(fieldStorage.file)
            if self.validate(result):
                blob_key = str(
                    self.write_blob(fieldStorage.value, result)
                )
                blob_keys.append(blob_key)
                result['deleteType'] = 'DELETE'
                result['deleteUrl'] = self.request.host_url +\
                    '/?key=' + urllib.quote(blob_key, '')
                if (IMAGE_TYPES.match(result['type'])):
                    try:
                        result['url'] = images.get_serving_url(
                            blob_key,
                            secure_url=self.request.host_url.startswith(
                                'https'
                            )
                        )
                        result['thumbnailUrl'] = result['url'] +\
                            THUMBNAIL_MODIFICATOR
                    except:  # Could not get an image serving url
                        pass
                if not 'url' in result:
                    result['url'] = self.request.host_url +\
                        '/' + blob_key + '/' + urllib.quote(
                            result['name'].encode('utf-8'), '')
            results.append(result)
        deferred.defer(
            cleanup,
            blob_keys,
            _countdown=EXPIRATION_TIME
        )
        return results

    def options(self):
        pass

    def head(self):
        pass

    def get(self):
        self.redirect(WEBSITE)

    def post(self):
        if (self.request.get('_method') == 'DELETE'):
            return self.delete()
        result = {'files': self.handle_upload()}
        s = json.dumps(result, separators=(',', ':'))
        redirect = self.request.get('redirect')
        if redirect:
            return self.redirect(str(
                redirect.replace('%s', urllib.quote(s, ''), 1)
            ))
        if 'application/json' in self.request.headers.get('Accept'):
            self.response.headers['Content-Type'] = 'application/json'
        self.response.write(s)

    def delete(self):
        key = self.request.get('key') or ''
        blobstore.delete(key)
        s = json.dumps({key: True}, separators=(',', ':'))
        if 'application/json' in self.request.headers.get('Accept'):
            self.response.headers['Content-Type'] = 'application/json'
        self.response.write(s)


class DownloadHandler(blobstore_handlers.BlobstoreDownloadHandler):
    def get(self, key, filename):
        if not blobstore.get(key):
            self.error(404)
        else:
            # Prevent browsers from MIME-sniffing the content-type:
            self.response.headers['X-Content-Type-Options'] = 'nosniff'
            # Cache for the expiration time:
            self.response.headers['Cache-Control'] = 'public,max-age=%d' % EXPIRATION_TIME
            # Send the file forcing a download dialog:
            self.send_blob(key, save_as=filename, content_type='application/octet-stream')

app = webapp2.WSGIApplication(
    [
        ('/', UploadHandler),
        ('/([^/]+)/([^/]+)', DownloadHandler)
    ],
    debug=True
)
